package com.bluetoothprinter_sp300.bluetooth;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Date;

import com.bluetoothprinter_sp300.Constant.MessageConstant;
import com.bluetoothprinter_sp300.activity.BaseActivity;
import com.bluetoothprinter_sp300.util.DimenCodeImageUtils;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class BlueToothPrintFunction {
	private static final String TAG = BlueToothPrintFunction.class
			.getSimpleName();
	private Handler handler;
	public static final byte BYTES_CMD_AUTO_FORWARD_SUCCESS = 10;
	public static final byte[] BYTES_CMD_FORWARD_AUTOMATIC = new byte[] { 27,
			28 };
	public static final byte[] BYTES_CMD_LABEL_DETECTION=new byte[]{0x1b,0x4e};
	//public static final byte[] BYTES_CMD_LABEL_DETECTION=new byte[]{0x1B, 0x40};
	boolean printFinish;
	public BlueToothPrintFunction(Handler handler) {
		this.handler = handler;
		
	}

	/**
	 * 打印机初始化
	 * <p>
	 * 执行打印机初始化操作，包括：
	 * <ul>
	 * <li>清除打印缓存区内容</li>
	 * <li>恢复默认值</li>
	 * </ul>
	 * 但该操作不清空数据接收缓存区中的数据。
	 * </p>
	 */
	public static enum PAPER_DETECT_PROFILE {
		/**
		 * 每点行检测缺纸
		 */
		EVERY_POINT_LINE,

		/**
		 * 每字符行检测缺纸
		 */
		EVERY_LINE,

		/**
		 * 内部不检测,外部调用者显性调用 {@link BlueToothZsdPosPrinterDriver#cmdDetectPaper()}
		 */
		NONE
	}

	/**
	 * 检测纸的频度
	 */
	private PAPER_DETECT_PROFILE detectPaper = PAPER_DETECT_PROFILE.NONE;
	public static final byte[] BYTES_CMD_INIT = { 27, 64 };

	/**
	 * 缺纸检测
	 * <p>
	 * 打印机在接收到该指令后，向上位机反馈一个字节的数据
	 * </p>
	 * <ul>
	 * <li> {@value #BYTE_RESPONE_HAS_PAPER} 有纸</li>
	 * <li> {@value #BYTE_RESPONE_NO_PAPER} 无纸</li>
	 * </ul>
	 */
	public static final byte[] BYTES_CMD_DETECT_PAPER = { 16, 4, 4 };

	/**
	 * 打印对比度设置
	 * <p>
	 * 该指令可根据不同的电源需求调整打印黑度，值的范围为 8 ~ 64。默认值 8
	 * </p>
	 */
	public static final byte[] BYTES_CMD_CONSTRAST = { 16, 5, 5 };

	/**
	 * 换行命令
	 * <p>
	 * 打印机收到换行命令后，打印出换行命令前输入的所有可打印的字符。如果之前未输入可打印的字符，打印机将空走纸一个字符行（只走纸，不打印）
	 * </p>
	 */
	public static final byte[] BYTES_CMD_LF = { 10 };

	/**
	 * 回车命令
	 * <p>
	 * 打印机收到换行命令后，打印出回车命令前输入的所有可打印的字符。如果之前未输入可打印的字符，打印机将空走纸一个字符行（只走纸，不打印）
	 * </p>
	 */
	public static final byte[] BYTES_CMD_CR = { 13 };

	/**
	 * 向前走纸n点行
	 * <p>
	 * 打印机向前空走纸N个点
	 * </p>
	 */
	public static final byte[] BYTES_CMD_FORWARD_N_POINT = { 27, 74 };

	/**
	 * 向前走纸N行空行
	 */
	public static final byte[] BYTES_CMD_FORWARD_N_LINE = { 27, 102, 1 };

	/**
	 * 退后走纸n点行
	 * <p>
	 * 打印机退后空走纸N个点
	 * </p>
	 */
	public static final byte[] BYTES_CMD_BACKWARD_N_POINT = { 27, 75 };

	/**
	 * 打开下划线打印
	 */
	public static final byte[] BYTES_CMD_ENABLE_UNDERLINE = { 27, 43, 1 };

	/**
	 * 关闭下划线打印
	 */
	public static final byte[] BYTES_CMD_DISABLE_UNDERLINE = { 27, 43, 0 };

	/**
	 * 设置行间距
	 * <p>
	 * 为随之而来的换行命令设置n点行间距，该值表示两个字符行之间的空白点行数。默认值是 3
	 * </p>
	 */
	public static final byte[] BYTES_CMD_LINESPACE = { 27, 49 };

	/**
	 * 设置反白色打印
	 */
	public static final byte[] BYTES_CMD_ENABLE_ANTI_WHITE = { 27, 105, 1 };

	/**
	 * 取消反白色打印
	 */
	public static final byte[] BYTES_CMD_DISABLE_ANTI_WHITE = { 27, 105, 0 };

	/**
	 * 字符横纵方向都放大（1～6）倍
	 * <p>
	 * 该命令之后的所有字符将以基本宽度的n倍和基本高度的n倍打印。最大倍数为6
	 * </p>
	 */
	public static final byte[] BYTES_CMD_INCREASE_FONT_SIZE = { 27, 87 };

	/**
	 * 字符横方向放大（1～6）倍
	 * <p>
	 * 该命令之后的所有字符将以基本宽度的n倍打印。最大倍数为6
	 * </p>
	 */
	public static final byte[] BYTES_CMD_INCREASE_FONT_HORIZONTAL_SIZE = { 27,
			85 };

	/**
	 * 字符纵方向放大（1～6）倍
	 * <p>
	 * 该命令之后的所有字符将以基本高度的n倍打印。最大倍数为6
	 * </p>
	 */
	public static final byte[] BYTES_CMD_INCREASE_FONT_VERTICAL_SIZE = { 27, 86 };

	/**
	 * 行间距默认值 {@value #BYTE_DEFAUTL_LINESPACE}点行
	 */
	public static final byte BYTE_DEFAUTL_LINESPACE = 3;

	/**
	 * 默认字体大小
	 */
	public static final byte BYTE_DEFAUTL_FONT_SIZE = 1;

	/**
	 * 有纸的反馈
	 */
	public static final byte BYTE_RESPONE_HAS_PAPER = 0;

	/**
	 * 无纸的反馈
	 */
	public static final byte BYTE_RESPONE_NO_PAPER = 108;

	public static final byte BYTE_RESPONE_4F = 79;

	/**
	 * 对比度默认值
	 */
	public static final byte BYTE_DEFAUTL_CONSTRAST = 8;
	/**
	 * 打印条形码
	 */
	public static final byte PRINT_BARCORD[] = { 30, 66 };

	/**
	 * 换行命令
	 * <p>
	 * 打印机收到换行命令后，打印出换行命令前输入的所有可打印的字符。如果之前未输入可打印的字符，打印机将空走纸一个字符行（只走纸，不打印）
	 * </p>
	 * 
	 * @throws IOException
	 */
	public void cmdLF() {
		pushCmd(BYTES_CMD_LF);
		if (PAPER_DETECT_PROFILE.EVERY_LINE.equals(detectPaper)) {
			pushCmd(BYTES_CMD_DETECT_PAPER);
		}
	}

	/**
	 * 回车命令
	 * <p>
	 * 打印机收到换行命令后，打印出回车命令前输入的所有可打印的字符。如果之前未输入可打印的字符，打印机将空走纸一个字符行（只走纸，不打印）
	 * </p>
	 * 
	 * @throws IOException
	 */
	public void cmdCR() {
		pushCmd(BYTES_CMD_CR);
		// pushCmd("\n".getBytes());
		if (PAPER_DETECT_PROFILE.EVERY_LINE.equals(detectPaper)) {
			pushCmd(BYTES_CMD_DETECT_PAPER);
		}
	}

	/**
	 * 向前空走纸N点行
	 * 
	 * @param n
	 *            N点行
	 * @throws IOException
	 */
	public void cmdForward(byte n) {
		pushCmd(BYTES_CMD_FORWARD_N_POINT);
		pushCmd(n);
		if (PAPER_DETECT_PROFILE.EVERY_POINT_LINE.equals(detectPaper)) {
			pushCmd(BYTES_CMD_DETECT_PAPER);
		}
	}

	/**
	 * 向前空走纸N行空行
	 * 
	 * @param n
	 *            N点行
	 * @throws IOException
	 */
	public void cmdForwardLine(byte n) {
		pushCmd(BYTES_CMD_FORWARD_N_LINE);
		pushCmd(n);
		if (PAPER_DETECT_PROFILE.EVERY_LINE.equals(detectPaper)) {
			pushCmd(BYTES_CMD_DETECT_PAPER);
		}
	}

	/**
	 * 退后空走纸N点行
	 * 
	 * @param n
	 *            N点行
	 * @throws IOException
	 */
	public void cmdBackward(byte n) {
		pushCmd(BYTES_CMD_BACKWARD_N_POINT);
		pushCmd(n);
		if (PAPER_DETECT_PROFILE.EVERY_POINT_LINE.equals(detectPaper)) {
			pushCmd(BYTES_CMD_DETECT_PAPER);
		}
	}

	/**
	 * 打开下划线打印
	 * 
	 * @throws IOException
	 */
	public void cmdEnableUnderLine() {
		pushCmd(BYTES_CMD_ENABLE_UNDERLINE);
	}

	/**
	 * 关闭下划线打印
	 * 
	 * @throws IOException
	 */
	public void cmdDisableUnderLine() {
		pushCmd(BYTES_CMD_DISABLE_UNDERLINE);
	}

	/**
	 * 恢复默认行间距
	 * 
	 * @throws IOException
	 */
	public void cmdLineSpace() {
		cmdLineSpace(BYTE_DEFAUTL_LINESPACE);
	}

	/**
	 * 设置行间距
	 * <p>
	 * 为随之而来的换行命令设置n点行间距，该值表示两个字符行之间的空白点行数。
	 * </p>
	 * 
	 * @param n
	 *            n点行间距
	 * @throws IOException
	 */
	public void cmdLineSpace(byte n) {
		pushCmd(BYTES_CMD_LINESPACE);
		pushCmd(n);
	}

	/**
	 * 设置反白色打印
	 * 
	 * @throws IOException
	 */
	public void cmdEnableAntiWhite() {
		pushCmd(BYTES_CMD_ENABLE_ANTI_WHITE);
	}

	/**
	 * 取消反白色打印（默认）
	 * 
	 * @throws IOException
	 */
	public void cmdDisableAntiWhite() {
		pushCmd(BYTES_CMD_DISABLE_ANTI_WHITE);
	}

	/**
	 * 字符横方向放大（1～6）倍
	 * <p>
	 * 该命令之后的所有字符将以基本宽度的n倍打印。最大倍数为6
	 * </p>
	 * @param n
	 * @throws IOException
	 */
	public void cmdIncreaseHorizontalFontSize(byte n) {
		pushCmd(BYTES_CMD_INCREASE_FONT_HORIZONTAL_SIZE);
		sendFontSize(n);

	}

	/**
	 * 字符纵方向放大（1～6）倍
	 * <p>
	 * 该命令之后的所有字符将以基本高度的n倍打印。最大倍数为6
	 * </p>
	 * @param n
	 * @throws IOException
	 */
	public void cmdIncreaseVerticalFontSize(byte n) {
		pushCmd(BYTES_CMD_INCREASE_FONT_VERTICAL_SIZE);
		sendFontSize(n);
	}

	/**
	 * 字符放大（1～6）倍
	 * <p>
	 * 该命令之后的所有字符将以基本宽、高度的n倍打印。最大倍数为6
	 * </p>
	 * 
	 * @param n
	 * @throws IOException
	 */
	public void cmdIncreaseFontSize(byte n) {

		pushCmd(composite(BYTES_CMD_INCREASE_FONT_SIZE, n));
	}

	private byte[] composite(byte[] bytes, byte b) {
		byte[] rst = new byte[bytes.length + 1];
		System.arraycopy(bytes, 0, rst, 0, bytes.length);
		rst[bytes.length] = b;
		return rst;
	}

	/**
	 * 重置字体大小
	 * <p>
	 * 该命令之后的所有字符将以正常的基本宽、高度打印。
	 * </p>
	 * 
	 * @throws IOException
	 */
	public void cmdResetFontSize() {
		cmdIncreaseFontSize(BYTE_DEFAUTL_FONT_SIZE);
	}

	/**
	 * 设置打印字符大小，最小值1，最大值6
	 * 
	 * @param n
	 * @throws IOException
	 */
	private void sendFontSize(byte n) {
		n = restictFontSize(n);
		pushCmd(n);
	}

	private byte restictFontSize(byte n) {
		if (n < 1)
			n = 1;
		if (n > 6)
			n = 6;
		return n;
	}

	/**
	 * 打印机初始化
	 * <p>
	 * 执行打印机初始化操作，包括：
	 * <ul>
	 * <li>清除打印缓存区内容</li>
	 * <li>恢复默认值</li>
	 * </ul>
	 * 但该操作不清空数据接收缓存区中的数据。
	 * </p>
	 */
	public void cmdInit() {
		pushCmd(BYTES_CMD_INIT);
	}

	/**
	 * 检测是否缺纸
	 */
	public void cmdDetectPaper() {
		pushCmd(BYTES_CMD_DETECT_PAPER);
	}

	/**
	 * 打印对比度设置
	 * <p>
	 * 该指令可根据不同的电源需求调整打印黑度，值的范围为 8 ~ 64。默认值 8
	 * </p>
	 * 
	 * @param n
	 */
	public void cmdContrast(byte n) {
		pushCmd(BYTES_CMD_CONSTRAST);
		pushCmd(n);
	}

	/**
	 * 恢复打印对比度
	 */
	public void cmdContrast() {
		pushCmd(BYTE_DEFAUTL_CONSTRAST);
	}

	/**
	 * 检测缺纸的方式
	 * 
	 * @return 检测缺纸的方式
	 */
	public PAPER_DETECT_PROFILE getDetectPaper() {
		return detectPaper;
	}

	/**
	 * 检测缺纸的方式
	 * 
	 * @param detectPaper
	 *            检测缺纸的方式
	 *            <ul>
	 *            <li>{@link PAPER_DETECT_PROFILE#EVERY_LINE}</li>
	 *            <li>{@link PAPER_DETECT_PROFILE#EVERY_POINT_LINE}</li>
	 *            <li>{@link PAPER_DETECT_PROFILE#NONE}</li>
	 *            </ul>
	 */
	public void setDetectPaper(PAPER_DETECT_PROFILE detectPaper) {
		this.detectPaper = detectPaper;
	}

	
	/**
	 * 整体测试
	 */
	public void printTest() {
		sendMessage("产品名称:便携式蓝牙打印机\\n" + "产品型号:SP300系列\n"
				+ "通信接口:蓝牙/USB/RS232\n" + "波 特 率:115200/可选\n"
				+ "设计生产:深圳市大通优联科技有限公司\n" + "服务网址:www.datongsmart.com \n");
		if (!isRecv(3500, "OK\r\n")) {
			return;
		}
		// Log.i(TAG,("我收到："+read());
		cmdForwardLine((byte) 1);
		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		sendMessage("以下为测试内容:\n1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ,.;:\"?/<>{}[]~!@#$%^&*()_-+=|\n"
				+ "  蓝牙，是一种支持设备短距离通信（一般10m内）的无线电技术。能在包括移动电话、PDA、无线耳机、笔记本电脑、相关外设等众多设备之间进行无线信息交换。利用“蓝牙”技术，能够有效地简化移动通信终端设备之间的通信，也能够成功地简化设备与因特网Internet之间的通信，从而数据传输变得更加迅速高效，为无线通信拓宽道路。蓝牙采用分散式网络结构以及快跳频和短包技术，支持点对点及点对多点通信，工作在全球通用的2.4GHz ISM（即工业、科学、医学）频段。其数据速率为1Mbps。采用时分双工传输方案实现全双工传输。\n");
		if (!isRecv(5500, "OK\r\n")) {
			return;
		}
		cmdForwardLine((byte) 1);
		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		cmdIncreaseHorizontalFontSize((byte) 1);
		sendMessage("字符横向放大");
		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		cmdIncreaseVerticalFontSize((byte) 1);
		sendMessage("字符纵向放大");
		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		cmdIncreaseFontSize((byte) 1);
		sendMessage("字符横纵向放大");

		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		cmdIncreaseHorizontalFontSize((byte) 2);
		sendMessage("字符横向放大2倍");

		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		cmdIncreaseVerticalFontSize((byte) 2);
		sendMessage("字符纵向放大2倍");

		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		cmdIncreaseFontSize((byte) 2);
		sendMessage("字符横纵向放大2倍");
		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
		sendMessage("打印条形码");

		if (!isRecv(1000, "OK\r\n")) {
			return;
		}
		byte[] datas = { 0x1E, 0x42, 0x34, 0x50, 0x32, 0x30, 0X0D, 0x36, 0X39,
				0X32, 0X38, 0X34, 0X32, 0X36, 0x34, 0x30, 0x30, 0x30, 0x36,
				0x35, 0x0A };
		pushCmd(datas);
		if (!isRecv(2500, "OK\r\n")) {
			return;
		}
		cmdForwardLine((byte) 1);
		if (!isRecv(1500, "OK\r\n")) {
			return;
		}
	}

	/**
	 * 打印条形码 
	 * @param type 条形码类型
	 * @param hight 条形码高度
	 * @param width 条形码宽度
	 * @param hri 条形码提示文字
	 * @param barcode 条形码byte数组
	 */
	public void printBarcode(byte type, byte hight, byte width, byte hri,
			byte[] barcode) {
		byte[] send = new byte[8 + barcode.length];
		System.arraycopy(PRINT_BARCORD, 0, send, 0, PRINT_BARCORD.length);
		send[PRINT_BARCORD.length] = type;
		send[PRINT_BARCORD.length + 1] = hight;
		send[PRINT_BARCORD.length + 2] = width;
		send[PRINT_BARCORD.length + 3] = hri;
		send[PRINT_BARCORD.length + 4] = (byte) barcode.length;
		System.arraycopy(barcode, 0, send, PRINT_BARCORD.length + 5,
				barcode.length);
		send[send.length - 1] = (byte) 10;
		pushCmd(send);
	}

	/**
	 * Sends a message.
	 * 
	 * @param message
	 *            A string of text to send.
	 * 
	 */
	public void sendMessage(String message) {

		// 检查实际要发送的东西
		if (message.length() > 0) {
			// 得到消息字节并且告诉BluetoothService
			byte[] send;
			try {
				send = message.getBytes("GB2312");
			} catch (UnsupportedEncodingException e) {
				send = message.getBytes();
			}
			pushCmd(send);
			pushCmd(BYTES_CMD_LF);

		}
	}
	/**
	 * 打印二维码
	 *@param codeValue:二维码的内容
	 *@param width二维码宽度
	 *@param height:二维码高度
	 *@param num 打印的份数
	 *@param cache 打印机缓存
	 *@param isHorizontal 是否横着打印
	 *@param isLabelDetection 是否标签检测
	 *@param colorThreshold 色彩值
	 * @throws IOException
	*/
	public void Print2DCode(String codeValue,int width,int height, int num, int cache,boolean isHorizontal,int colorThreshold,boolean isLabelDetection)
	{
	Bitmap codeBmp=DimenCodeImageUtils.createTwoDimenCodeImage(codeValue, width, height);
	printBmp(codeBmp,num,cache,isHorizontal,colorThreshold,isLabelDetection);
	}

	/**
	 * 打印bmp图片
	 *@param bm bmp图片
	 *@param num 打印的份数
	 *@param cache 打印机缓存
	 *@param isHorizontal 是否横着打印
	 *@param isLabelDetection 是否标签检测
	 *@param colorThreshold 色彩值
	 * @throws IOException
	 */
	public void printBmp(Bitmap bm, int num, int cache,boolean isHorizontal,int colorThreshold,boolean isLabelDetection) {
		if (bm == null) {
			return;
		}
		System.out.println("开始宽高：" + bm.getWidth() + ":" + bm.getHeight());

		Rect src = new Rect();
		Rect dst = new Rect();

		if (!isHorizontal) {
			bm.prepareToDraw();
			Matrix matrix = new Matrix();
			matrix.postRotate(90);
			bm = Bitmap.createBitmap(bm, 0, 0, bm.getWidth(), bm.getHeight(),
					matrix, true);

			System.out.println("旋转后宽高：" + bm.getWidth() + ":" + bm.getHeight());

			if (bm.getWidth() > 376f) {
				src.left = bm.getWidth() - 376;
			} else {
				src.left = 0;

			}
			src.top = 0;
			src.right = bm.getWidth();
			src.bottom = bm.getHeight();

			if (bm.getWidth() > 376f)
			{
			dst.left =0;
			}else
			{
				dst.left=376-bm.getWidth();
			}
			dst.top = 0;
			dst.right = 376;
			dst.bottom = bm.getHeight();

		} else {
			bm.prepareToDraw();
			src.left = 0;
			src.top = 0;
			if (bm.getWidth() > 376f) {
				src.right = 376;
			} else {
				src.right = bm.getWidth();
			}
			
			src.bottom = bm.getHeight();
			dst.left = 0;
			dst.top = 0;
			if (bm.getWidth() > 376f) {
				dst.right = 376;
			} else {
				dst.right = bm.getWidth();
			}
			dst.bottom = bm.getHeight();

		}
		Bitmap bmp = Bitmap.createBitmap(376, bm.getHeight(),
				Bitmap.Config.ARGB_8888);

		int[] pix = new int[376 * bm.getHeight()];
		for (int y = 0; y < bm.getHeight(); y++)
			for (int x = 0; x < 376; x++) {
				int index = y * 376 + x;
				int r = ((pix[index] >> 16) & 0xff) | 0xff;
				int g = ((pix[index] >> 8) & 0xff) | 0xff;
				int b = (pix[index] & 0xff) | 0xff;
				pix[index] = 0xff000000 | (r << 16) | (g << 8) | b;
				// pix[index] = 0xff000000;

			}
		bmp.setPixels(pix, 0, 376, 0, 0, 376, bmp.getHeight());
		Canvas canvas = new Canvas(bmp);
		canvas.drawBitmap(bm, src, dst, null);
		bmp.prepareToDraw();
		Log.i(TAG, bmp.getWidth() + " / " + bmp.getHeight() + " "
				+ bmp.getConfig().toString());
		int[] colors = new int[bmp.getWidth() * bmp.getHeight()];
		bmp.getPixels(colors, 0, bmp.getWidth(), 0, 0, bmp.getWidth(),
				bmp.getHeight());
		byte[] res = new byte[bmp.getWidth() / 8 * bmp.getHeight()];
		for (int i = 0; i < res.length; i++) {
			int offset = i * 8;
			byte bit = 0;
			for (int j = 0; j < 8; j++) {
				int c = colors[offset + j];

				byte r = (byte) ((c >>> 16) & 127);
				byte g = (byte) ((c >>> 8) & 127);
				byte b = (byte) (c & 127);
				double gray = r * 0.3 + g * 0.5 + b * 0.11;
        
				if (gray < colorThreshold) {
					bit |= (1 << (7 - j));
				}
				if (c == -1) {
					res[i] = 127;
				} else {
					res[i] = 0;
				}
			}
			res[i] = bit;

		}
		sendBitmap(res, bmp.getWidth(), bmp.getHeight(), num,cache,isLabelDetection);
	}
	/**
	 * 发送bmp字节数据
	 *@param bytes bmp字节数据
	 *@param width 图片的宽度
	 *@param height 图片的高度
	 *@param num 打印的份数
	 *@param cache 打印机缓存
	 *@param isLabelDetection 是否标签检测
	 * @throws IOException
	 */
	private void sendBitmap(final byte[] bytes, final int width, final int height,
			final int num,final int cache,final boolean isLabelDetection) {

		Thread thread = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int index = 0; index < num; index++) {
					handler.obtainMessage(
							MessageConstant.MESSAGE_PRINT_PERCENT, index + 1,
							num).sendToTarget();
					byte[] head = { 28, 57, 0x77, 0x01, 20, 00, 00 };
					byte BmpArr[] = new byte[cache * 47];
					int n;
					int len, i;
					short cnt = 0;
					n = 0;
					len = width / 8;
					if (width % 8 > 0)
						len += 1;
					Log.i(TAG, "长度：" + bytes.length);
					//
					while (n < height) {
						cnt = 0;
						for (i = 0; i < cache; i++) {
							System.arraycopy(bytes, n * len, BmpArr, i * 47,
									len);
							n++;
							cnt++;
							if (n >= height)
								break;
						}
						head[4] = getLowWord(cnt);
						head[5] = getHighWord(cnt);
						
						pushCmd(head);
						byte uu[] = new byte[cnt * 47];
						System.arraycopy(BmpArr, 0, uu, 0, cnt * 47);
						pushCmd(uu);
						pushCmd((byte) 13);
						//pushCmd(new byte[] { 16, 5, 5, 40 });
						long time=10000;
						if(cache>500)
						{
							time=50000;
						}
						if (!isRecv(time, "m")) {
							
							Message msg = handler.obtainMessage(MessageConstant.MESSAGE_TOAST);
							Bundle bundle = new Bundle();
							bundle.putString(MessageConstant.TOAST, "打印超时！");
							msg.setData(bundle);
							handler.sendMessage(msg);
							
							return;
						}
						n++;
					}
  					if(isLabelDetection)
					{
						pushCmd(BYTES_CMD_LABEL_DETECTION);
						boolean isRecvOK=isRecv(10000,"OK\r\n")||isRecv(10000,"ONPAPER\r\n")||isRecv(10000,"ERROR\r\n");
						if (!isRecvOK) {
						
							Message msg = handler.obtainMessage(MessageConstant.MESSAGE_TOAST);
							Bundle bundle = new Bundle();
							bundle.putString(MessageConstant.TOAST, "检测标签错误！");
							msg.setData(bundle);
							handler.sendMessage(msg);

							return;
						}
					}
				}  

				handler.obtainMessage(MessageConstant.MESSAGE_PRINT_END)
				.sendToTarget();
			}
		});
		thread.start();

	}
	/**
	 * 给打印机发送一个byte字节
	 * @param n byte字节
	 */

	public void pushCmd(byte[] bytes) {
			BaseActivity.mService.pushCmd(bytes);
	}

	public void pushCmd(byte n) {
		BaseActivity.mService.pushCmd(n);
	}
	/**
	 * 判断打印是否超时
	 * @param timeout 设定的超时时间
	 * @param str 打印机返回的
	 */

	public boolean isRecv(long timeout,String str) {
		Date dt = new Date();
		Long time = dt.getTime();
		Long time1;
		

			while (BaseActivity.mService.returnStr.indexOf(str)==-1) {
				dt = new Date();
				time1 = dt.getTime();
				if (time1 - time > timeout) {
					Log.i(TAG, "time out.....");
					break;
				}
			}
			return BaseActivity.mService.returnStr.indexOf(str)!=-1;
		

	}
	/**
	 * 取short类型的高字节
	 * @param intval short数据
	 */

	 public byte getHighWord(short intval)
    {
        return (byte)(intval >>>8);
    }
	 /**
		 * 取short类型的低字节
		 * @param intval short数据
		 */

     public byte getLowWord(short intval)
    {
        return (byte) (intval& 0xff);
    }
}
